Method: Merb::AbstractController._template_root=

Defined in:
lib/merb-core/controller/abstract_controller.rb

._template_root=(root) ⇒ Object

Resets the template roots to the template root passed in.

Parameters

root<~to_s>

The new path to set the template root to.

:api: public



195
196
197
198
# File 'lib/merb-core/controller/abstract_controller.rb', line 195

def self._template_root=(root)
  @_template_root = root
  _reset_template_roots
end