Method: Merb::Test::RequestHelper#delete
- Defined in:
- lib/merb-core/test/helpers/mock_request_helper.rb
#delete(path, params = {}, env = {}, &block) ⇒ Object
Deprecated.
An HTTP DELETE request that operates through the router
Parameters
- path<String>
-
The path that should go to the router as the request uri.
- params<Hash>
-
An optional hash that will end up as params in the controller instance.
- env<Hash>
-
An optional hash that is passed to the fake request. Any request options should go here (see fake_request).
- &blk
-
The controller is yielded to the block provided for actions prior to the action being dispatched.
:api: public
296 297 298 299 |
# File 'lib/merb-core/test/helpers/mock_request_helper.rb', line 296 def delete(path, params = {}, env = {}, &block) env[:request_method] = "DELETE" mock_request(path, params, env, &block) end |